//
//  RootViewController.m
//  Impressionist
//
//  Created by Stephanie Abascal on 1/24/09.
//  Copyright Stephanie Abascal 2009. All rights reserved.
//

#import "RootViewController.h"
#import "MainViewController.h"
#import "FlipsideViewController.h"


@implementation RootViewController

@synthesize infoButton;
@synthesize flipsideNavigationBar;
@synthesize mainViewController;
@synthesize flipsideViewController;


- (void)viewDidLoad {
    
    [super viewDidLoad];
    MainViewController *viewController = [[MainViewController alloc] initWithNibName:@"MainView" bundle:nil];
    self.mainViewController = viewController;
    [viewController release];
    
	[self loadFlipsideViewController];
	
    [self.view insertSubview:mainViewController.view belowSubview:infoButton];
	adView.adCode = @"12345678-90ab-cdef-1234-567890abcedf";
	adView.frame = CGRectMake(10.0, -50.0, 300.0, 50.0);
	adVisible = NO;
	
	/*[NSTimer scheduledTimerWithTimeInterval:30.0
									 target:self
								   selector:@selector(toggleAd:)
								   userInfo:nil
									repeats:NO];*/
	
}


- (void)loadFlipsideViewController {
    
    FlipsideViewController *viewController = [[FlipsideViewController alloc] initWithNibName:@"FlipsideView" bundle:nil];
    self.flipsideViewController = viewController;
    [viewController release];
	viewController.mainController = mainViewController;
	viewController.rootViewController = self;
    
    // Set up the navigation bar
    UINavigationBar *aNavigationBar = [[UINavigationBar alloc] initWithFrame:CGRectMake(0.0, 0.0, 320.0, 44.0)];
    aNavigationBar.barStyle = UIBarStyleBlackOpaque;
    self.flipsideNavigationBar = aNavigationBar;
    [aNavigationBar release];
    
    UIBarButtonItem *buttonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(toggleView)];
    UINavigationItem *navigationItem = [[UINavigationItem alloc] initWithTitle:@"Impressionist"];
    navigationItem.rightBarButtonItem = buttonItem;
    [flipsideNavigationBar pushNavigationItem:navigationItem animated:NO];
    [navigationItem release];
    [buttonItem release];
}


- (IBAction)toggleView {    
    /*
     This method is called when the info or Done button is pressed.
     It flips the displayed view from the main view to the flipside view and vice-versa.
     */
    if (flipsideViewController == nil) {
        [self loadFlipsideViewController];
    }
    
    UIView *mainView = mainViewController.view;
    UIView *flipsideView = flipsideViewController.view;
    
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:1];
    [UIView setAnimationTransition:([mainView superview] ? UIViewAnimationTransitionFlipFromRight : UIViewAnimationTransitionFlipFromLeft) forView:self.view cache:YES];
    
    if ([mainView superview] != nil) {
        [flipsideViewController viewWillAppear:YES];
        [mainViewController viewWillDisappear:YES];
        [mainView removeFromSuperview];
        [infoButton removeFromSuperview];
		[adView removeFromSuperview];
        [self.view addSubview:flipsideView];
        [self.view insertSubview:flipsideNavigationBar aboveSubview:flipsideView];
        [mainViewController viewDidDisappear:YES];
        [flipsideViewController viewDidAppear:YES];

    } else {
        [mainViewController viewWillAppear:YES];
        [flipsideViewController viewWillDisappear:YES];
        [flipsideView removeFromSuperview];
        [flipsideNavigationBar removeFromSuperview];
        [self.view addSubview:mainView];
        [self.view insertSubview:infoButton aboveSubview:mainViewController.view];
		[self.view insertSubview:adView aboveSubview:mainViewController.view];
        [flipsideViewController viewDidDisappear:YES];
        [mainViewController viewDidAppear:YES];
    }
    [UIView commitAnimations];
}

- (void)toggleNavBar {
	if ([flipsideNavigationBar superview])
		[flipsideNavigationBar removeFromSuperview];
	else
		[self.view insertSubview:flipsideNavigationBar aboveSubview:flipsideViewController.view];
}

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
    // Release anything that's not essential, such as cached data
}


- (void)dealloc {
    [infoButton release];
    [flipsideNavigationBar release];
    [mainViewController release];
    [flipsideViewController release];
    [super dealloc];
}

- (void)saveToAlbum {
	[self.mainViewController saveToAlbum];
}

- (void) adViewDidFinishLoad:(MobclixAdView *) adV {
	[UIView beginAnimations:@"fadein" context:nil];
	[UIView setAnimationDuration:1.0f];
	[UIView setAnimationCurve:UIViewAnimationCurveLinear];
	[UIView setAnimationDelegate:self];
	// Change any animatable properties here
	adView.frame = CGRectMake(10.0, 0.0, 300.0, 50.0);
	[UIView commitAnimations];
	
	adVisible = YES;
	
	[NSTimer scheduledTimerWithTimeInterval:60.0
									 target:self
								   selector:@selector(toggleAd:)
								   userInfo:nil
									repeats:NO];
}

- (void) adViewDidFailLoad: (MobclixAdView *) adV {
	[adView getAd];
}

- (BOOL) adViewShouldTouchThrough: (MobclixAdView *) adV {
	return adVisible;
}

- (void)toggleAd:(NSTimer*)theTimer {
	if (!adVisible) {
		[adView getAd];
	} else {
		[UIView beginAnimations:@"fadeout" context:nil];
		[UIView setAnimationDuration:1.0f];
		[UIView setAnimationCurve:UIViewAnimationCurveLinear];
		[UIView setAnimationDelegate:self];
		// Change any animatable properties here
		adView.frame = CGRectMake(10.0, -50.0, 300.0, 50.0);
		[UIView commitAnimations];
		
		adVisible = NO;
		
		[NSTimer scheduledTimerWithTimeInterval:60.0
										 target:self
									   selector:@selector(toggleAd:)
									   userInfo:nil
										repeats:NO];
	}
}

@end
